{{
'-------------------------------------------------------------------------------
'                         !!! IMPORTANT NOTE !!!
'                         ======================
'    If the name of this file does not end in "_Input.spin", then this
'    is not the original file - it is a processed version that will get 
'    overwritten by the next compile. Do not change this file - instead, 
'    locate the original version (normally in the 'input' subdirectory of 
'    the Catalina target directory) and modify that version of this file.
'-------------------------------------------------------------------------------
'
' Catalina_Common - Common definitions for the Catalina LMM and XMM Kernels and
'                   targets.
' 
'                   This file is where the PIN Definitions used by various
'                   Propeller devices are stored. It also contains routines
'                   for setting up and using the common registry.
'
'
'                   This is a CUSTOM version of this file. It contains only
'                   a default set of definitions (suitable for the HYDRA), and 
'                   another set activated by defining the symbol CUSTOM (but 
'                   which requires editing!)
'
' Version 2.8 - Initial version - by Ross Higson
'
'------------------------------------------------------------------------------
'
'    Copyright 2009 Ross Higson
'
'    This file is part of the Catalina Target Package.
'
'    The Catalina Target Package is free software: you can redistribute 
'    it and/or modify it under the terms of the GNU Lesser General Public 
'    License as published by the Free Software Foundation, either version 
'    3 of the License, or (at your option) any later version.
'
'    The Catalina Target Package is distributed in the hope that it will
'    be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
'    of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
'    See the GNU Lesser General Public License for more details.
'
'    You should have received a copy of the GNU Lesser General Public 
'    License along with the Catalina Target Package.  If not, see 
'    <http://www.gnu.org/licenses/>.
'
'------------------------------------------------------------------------------
}}
CON
'
' Pin groups or base pins for mouse, keyboard and screen. Change these to suit
' your Propeller platform. Add more definitions here if required for additional
' platforms, drivers and plugins.
'
' Clock modes and crystal frequency must be set in the top level object, but
' those objects just use the definitions here. Change them as required to suit
' your Propeller platform.
'
#ifdef CUSTOM
'
' Comment out the following line when you have configured the Custom platform:
'
ERROR : CUSTOM PLATFORM HAS NOT BEEN CONFIGURED YET!

'===============================================================================
'
' Custom platform General definitions:
' 
'===============================================================================

KBD_PIN    = -1                 ' BASE PIN  (Custom)
MOUSE_PIN  = -1                 ' BASE PIN  (Custom)
TV_PIN     = -1                 ' BASE PIN  (Custom)
VGA_PIN    = -1                 ' BASE PIN  (Custom)
SD_DO_PIN  = -1                 ' Custom has no SD Card
SD_CLK_PIN = -1                 ' Custom has no SD Card 
SD_DI_PIN  = -1                 ' Custom has no SD Card
SD_CS_PIN  = -1                 ' Custom has no SD Card
I2C_PIN    = 28                 ' I2C Boot EEPROM SCL Pin
I2C_DEV    = $A0                ' I2C Boot EEPROM Device Address
SI_PIN     = 31                 ' PIN (Custom)
SO_PIN     = 30                 ' PIN (Custom)
'
' Custom platform Clock definitions:
' 
CLOCKMODE = xtal1 + pll16x      ' (Custom)
XTALFREQ  = 5_000_000           ' (Custom)
CLOCKFREQ = 80_000_000          ' (Custom) Nominal clock frequency (required by some drivers)
'
' Custom platform Loader and InterProp Comms definitions:
'
SIO_BAUD      = 115200          ' Baud rate to use for all interprop comms                 
SIO_LOAD_MODE = %0000           ' Custom has only one CPU - this is for serial loading
SIO_COMM_MODE = %0000           ' Custom has only one CPU - this is for serial loading
TX_PIN        = -1              ' Custom has only one CPU
RX_PIN        = -1              ' Custom has only one CPU
'
' Custom platform XMM Base Address:
'
XMM_BASE_ADDRESS = -1           ' Custom board has no XMM
'
' Custom platform Payload pins (SI/SO pins)
'
PAYLOAD_RXPIN = SI_PIN          ' Rx pin to use for Payload comms
PAYLOAD_TXPIN = SO_PIN          ' Tx pin to use for Payload comms
'
' Custom platform BlackCat debug pins (SI/SO pins)
'
BLACKCAT_RXPIN = SI_PIN         ' Rx pin to use for BlackCat comms
BLACKCAT_TXPIN = SO_PIN         ' Tx pin to use for BlackCat comms
BLACKCAT_MODE  = %0000          ' SIO mode to use for BlackCat comms
'
#else
'===============================================================================
'
' Default (Hydra) General definitions:
'
'===============================================================================
KBD_PIN    = 13                 ' BASE PIN  (Hydra)
MOUSE_PIN  = 2                  ' PIN GROUP (Hydra)
TV_PIN     = 24                 ' BASE PIN  (Hydra)
VGA_PIN    = 16                 ' BASE PIN  (Hydra)
SD_DO_PIN  = 16                 ' (Hydra) - note shared with VGA
SD_CLK_PIN = 17                 ' (Hydra) - note shared with VGA 
SD_DI_PIN  = 18                 ' (Hydra) - note shared with VGA
SD_CS_PIN  = 19                 ' (Hydra) - note shared with VGA
I2C_PIN    = 28                 ' I2C Boot EEPROM SCL Pin
I2C_DEV    = $A0                ' I2C Boot EEPROM Device Address
SI_PIN     = 31                 ' PIN (Hydra)
SO_PIN     = 30                 ' PIN (Hydra)
'
' Default (Hydra) Clock definitions:
' 
CLOCKMODE = xtal1 + pll8x       ' (Hydra)
XTALFREQ  = 10_000_000          ' (Hydra)
CLOCKFREQ = 80_000_000          ' (Hydra) Nominal clock frequency (required by some drivers)
'
' Default (Hydra) Loader and InterProp Comms definitions:
'
SIO_BAUD      = 115200          ' Baud rate to use for all interprop comms and serial loading                 
SIO_LOAD_MODE = %0000           ' Hydra has only one CPU - this is for serial loading
SIO_COMM_MODE = %0000           ' Hydra has only one CPU - this is for serial loading
TX_PIN        = -1              ' Hydra has only one CPU
RX_PIN        = -1              ' Hydra has only one CPU
'
'Default (Hydra) XMM Base Address:
'
XMM_BASE_ADDRESS = $0000_0000   ' XMM adddressing from 0
'
' Default (Hydra) Payload pins (SI/SO pins)
'
PAYLOAD_RXPIN = SI_PIN          ' Rx pin to use for Payload comms
PAYLOAD_TXPIN = SO_PIN          ' Tx pin to use for Payload comms
'
' Default (Hydra) BlackCat debug pins (mouse pins) - can be used for LMM or XMM but requires special cable
'
BLACKCAT_RXPIN = 9              ' Rx pin to use for BlackCat comms
BLACKCAT_TXPIN = 10             ' Tx pin to use for BlackCat comms
BLACKCAT_MODE  = %0010          ' SIO must invert Tx on Hydra when using mouse port
{
'
' Default (Hydra) BlackCat debug pins (SI/SO pins) - can only be used for LMM, but uses normal serial cable
'
BLACKCAT_RXPIN = SI_PIN         ' Rx pin to use for BlackCat comms
BLACKCAT_TXPIN = SO_PIN         ' Tx pin to use for BlackCat comms
BLACKCAT_MODE  = %0000          ' SIO mode to use for BlackCat comms
}
'
#endif
'===============================================================================
'
' For TV modes, select PAL_NTSC, INTERLACE
'
#ifdef NTSC
NTSC_PAL = 0                    ' 0 = NTSC                       
#else
NTSC_PAL = 1                    ' 1 = PAL
#endif

#ifdef NO_INTERLACE
INTERLACE = 0                   ' 0 = non-interlace
#else
INTERLACE = 1                   ' 1 = interlace
#endif
'
' Stack size required during SPIN initialization. May need to be increased
' if you add a lot of SPIN initialization code. This has nothing to do with
' the Catalina stack size.
'
STACKSIZE = 50
'
' Plugin types - the Catalina Kernel generally attempts to locate plugins
' by type, not by the cog they happen to be running on. Values up to 8
' are reserved. New plugin types should start from 9 onwards.
'
LMM_VMM = 0       ' main LMM (single threaded)
LMM_HMI = 1       ' HMI Drivers (Kbd/Mouse/Screen)
LMM_LIB = 2       ' Utility library
LMM_FLA = 3       ' Floating Point Library A
LMM_FLB = 4       ' Floating Point Library B
LMM_RTC = 5       ' Real-Time Clock
LMM_FIL = 6       ' File System
LMM_SIO = 7       ' Serial I/O
LMM_DUM = 8       ' Dummy Plugin
'
' Registry definitions. The Catalina registry provides a place where all
' plugins can be found, and a request block for sending requests to each
' plugin. The registry location must be known to all plugins. To use the
' registry, each plugin finds its own cog number, and then registers by
' putting its plugin type in the top 8 bits of the long at
'    long[REGISTRY][plugin_cog]
' The bottom 24 bits of that long point to the request block the plugin
' must use to receive requests and return results. Each request block
' is two longs - the first long is the request, which may be a 'short' 
' request (if the request and all its parameters can fit into a single
' long, or the address of a 'long' parameter block elsewhere in RAM. The
' plugin must know how to interpret these requests. When complete, the
' plusing zeroes the first long of the request block, and may either
' return the result in the second long of the request block (typical for
' 'short' requests, but also used for some 'long' requests), or in the
' 'long' parameter block.  
' 
COGSTORE     = $7EFC           ' long used for interacting with cog used to store command line
REGISTRY_END = COGSTORE        ' this location is fairly arbitrary.
REGISTRY     = REGISTRY_END - 4 * 1 * 8
REQUESTS     = REGISTRY     - 4 * 2 * 8
'
' SD Card Loader definitions. This determines where the cluster list
' of the file to be loaded will be located, and also the maximum size
' of this list. Note that currently, we assume FAT16, which means 
' each entry on the list is 16 bits.
'
FLIST_ADDRESS  = $7F00
MAX_FLIST_SIZE = ($8000 - FLIST_ADDRESS)/2
'
' SD Card Buffers and other File Load data. Used during program load 
' Must not overlap the registry. Programs to be loaded must therefore
' be less than FLIST_BUFF in size (i.e. 31kb). 
'
FLIST_SSIZ = 512                ' size of sector
FLIST_BUFF = $8000 - (2 * FLIST_SSIZ) ' sector buffer to use for load
FLIST_PREG = FLIST_BUFF + FLIST_SSIZ ' initial registry address
FLIST_PSTK = FLIST_PREG + 4     ' initial stack address
FLIST_FSIZ = FLIST_PSTK + 4     ' size of file to load 
FLIST_SHFT = FLIST_FSIZ + 4     ' log (base 2) of sectors per cluster
FLIST_SECT = FLIST_SHFT + 4     ' sector start of data region
FLIST_XFER = FLIST_SECT + 4     ' xfer buffer to use while loading
FLIST_SIOB = FLIST_XFER + 8     ' serial i/o buffer to use while loading
FLIST_END  = FLIST_XFER + 68    ' next available memory after load data.
'
' Floating point plugin definitions. The Transfer address points to
' 2 longs used to transfer requests between Catalina kernels without
' built-in floating point support and the Float_A plugin. Note that
' this can use the same location as the FLIST_ADDRESS above, since
' that is only used at load time, not run-time.
' 
FLOAT_XFER = $7F00
'
' Proxy Transfer addresses - required by some proxy drivers (2 longs)
'
PROXY_XFER = $7F08
'
' BlackCat Debugger Support (NOTE: must match the ones in Catalina_blackcat.s)
'
DEBUG_BREAK   = $7F10           ' Non-zero when at breakpoint and executing debug support code
DEBUG_ADDR    = $7F14           ' Holds address in kernel for a host requested read or write
DEBUG_OUT     = $7F18           ' Holds information written out from kernel space
DEBUG_IN      = $7F1C           ' Holds information to be written into kernel space
DEBUG_FLAG    = $7F20           ' Indicates read operation requested when non-zero
DEBUG_OVERLAY = LMM_INIT_OFF    ' Cog location to overlay debug code
DEBUG_VECTORS = $1eb            ' Cog location to overlay hub vectors (last 5 longs!!!)
DEBUG_MARKER  = LMM_PC_OFF      ' Cog location to write to DEBUG_BREAK at breakpoint                    
DEBUG_END     = $7F2F           ' Last hub address reseved for debugger
'
BLACKCAT_SIZE = 5 * 4           ' size (in bytes) of Hub RAM communications block for Black Cat
'
'BLACKCAT_BAUD = 9600            ' Baud Rate for Black Cat comms (RS232 or USB)
'BLACKCAT_BAUD = 57600           ' Baud Rate for Black Cat comms (RS232or USB)
BLACKCAT_BAUD = 115200          ' Baud Rate for Black Cat comms (RS232 or USB)
'BLACKCAT_BAUD = 230400          ' Baud Rate for Black Cat comms (USB)
'BLACKCAT_BAUD = 460800          ' Baud Rate for Black Cat comms (USB)
'BLACKCAT_BAUD = 921600          ' Baud Rate for Black Cat comms (USB)
'
' ARGC/ARGV Support
'
ARGC_ADDR     = $7F30           ' WORD argc goes here (will be loaded during initialization)
ARGV_ADDR     = $7F32           ' WORD address of argv must be stored here
ARGV_0        = $7F34           ' LONG default argv array starts here (up to ARGV_MAX entries)
ARGV_MAX      = 32              ' maximum size (in LONGs) of default argv array
'
' Kernel Constants. These values are used by the various targets
' and the POD debugger and disassembler as well as by the Kernel.
'
' NOTE : if the Kernel is changed, the values below must also
'        be changed or the debug targets will not work correctly. 
'
LMM_FIRST_REG_OFF = $2b                           
LMM_PC_OFF        = $2b
LMM_SP_OFF        = $2c
LMM_FP_OFF        = $2d
LMM_RI_OFF        = $2e
LMM_BC_OFF        = $2f
LMM_BA_OFF        = $30
LMM_BZ_OFF        = $31
LMM_CS_OFF        = $32
LMM_LAST_REG_OFF  = $4a
'
LMM_INIT_BZ_OFF   = $51
LMM_INIT_PC_OFF   = $52
'
LMM_INIT_OFF      = $53
'
#ifdef SMALL
LMM_1_OFF         = $72         ' for SMALL (XMM) Kernel
#elseifdef LARGE
LMM_1_OFF         = $72         ' for LARGE (XMM) Kernel
#else
LMM_1_OFF         = $67         ' for TINY (LMM) Kernel
LMM_2_OFF         = $6a
LMM_3_OFF         = $6d
LMM_4_OFF         = $70
#endif
'
LMM_FIRST_OPCODE  = $02
LMM_INIT          = $02
LMM_LODL          = $03
LMM_LODA          = $04
LMM_LODF          = $05
LMM_PSHL          = $06
LMM_PSHB          = $07
LMM_CPYB          = $08
LMM_NEWF          = $09
LMM_RETF          = $0a
LMM_CALA          = $0b
LMM_RETN          = $0c
LMM_CALI          = $0d
LMM_JMPA          = $0e
LMM_JMPI          = $0f
LMM_DIVS          = $10
LMM_DIVU          = $11
LMM_MULT          = $12
LMM_BR_Z          = $13
LMM_BRNZ          = $14
LMM_BRAE          = $15
LMM_BR_A          = $16
LMM_BRBE          = $17
LMM_BR_B          = $18
LMM_SYSP          = $19
LMM_PSHA          = $1a
LMM_FADD          = $1b
LMM_FSUB          = $1c
LMM_FMUL          = $1d
LMM_FDIV          = $1e
LMM_FCMP          = $1f
LMM_FLIN          = $20
LMM_INFL          = $21
LMM_PSHM          = $22
LMM_POPM          = $23
LMM_PSHF          = $24
LMM_RLNG          = $25
LMM_RWRD          = $26
LMM_RBYT          = $27
LMM_WLNG          = $28
LMM_WWRD          = $29
LMM_WBYT          = $2a
LMM_LAST_OPCODE   = $2a
'
' Some generally useful constants:
' 
#ifdef PC
HMI_PC  = TRUE
HMI_VGA = FALSE
HMI_TV  = FALSE
#elseifdef VGA
HMI_VGA = TRUE
HMI_PC  = FALSE
HMI_TV  = FALSE
#else
HMI_TV  = TRUE
HMI_PC  = FALSE
HMI_VGA = FALSE
#endif

#ifdef HIRES_VGA
#define HIRES = TRUE
#elseifdef HIRES_TV
#define HIRES = TRUE
#else
#define HIRES = FALSE
#endif
'
PUB InitializeRegistry | i
'
' this routine should be called by the target before it
' starts or initializes any of the plugins to be loaded
'
  repeat i from 0 to 7
    long[REGISTRY][i] := REQUESTS + (8*i)
    long[REQUESTS][2 * i] := 0
    long[REQUESTS][2 * i + 1] := 0

PUB Register (my_cog, my_plugin_type)
'
' this routine can be used by a plugin to register itself
' with the Catalina Kernel. Plugins must register themselves
' before the Kernel can invoke them or Catalina programs
' can use the functions they provide.
'
  long[REGISTRY][my_cog] := (my_plugin_type<<24) + (long[REGISTRY][my_cog] & $00FFFFFF)

PUB UnRegister (my_cog)
'
' this routine can be used by a plugin to unregister itself
' with the Catalina Kernel. Plugins must unregister themselves
' if they may later be stopped or restarted in a different cog.
'
  long[REGISTRY][my_cog] := (long[REGISTRY][my_cog] & $00FFFFFF)

PUB SendInitializationData(to_cog, data_1, data_2)
'
' this routine can be called to send initialization data
' to a cog - note that once initialization is complete,
' SPIN methods such as this cannot be used - requests
' must be done in PASM. Also note that since most
' plugins trigger off the first long of the request,
' it is important to initialize that long second.
'
  long[REQUESTS][2 * to_cog + 1] := data_2
  long[REQUESTS][2 * to_cog]     := data_1

PUB WaitForRequest(to_cog)
  ' wait for a request to be processed
  repeat while long[REQUESTS][2 * to_cog] <> 0
 
